@@ -1,25 +1,25 @@
 use std::{collections::HashMap, time::Duration};
 
-/*pub trait DefaultResponse {
+pub trait DefaultResponse {
     fn default() -> ResponseData;
-}*/
+}
 
 #[derive(Debug)]
 pub struct ResponseData {
     pub text: String,
     pub code: u16,
-    pub reflected_params: Vec<String>,
+    pub reflected_params: HashMap<String, usize>,
 }
 
-/*impl DefaultResponse for ResponseData {
+impl DefaultResponse for ResponseData {
     fn default() -> ResponseData {
         ResponseData {
             text: String::new(),
             code: 0u16,
-            reflected_params: vec![],
+            reflected_params: HashMap::new(),
         }
     }
-}*/
+}
 
 #[derive(Debug, Clone)]
 pub struct FuturesData {
@@ -58,7 +58,7 @@ pub struct Config {
     pub as_body: bool,
     pub headers_discovery: bool,
     pub within_headers: bool,
-    pub verbose: u8,
+    pub verbose: usize,
     pub is_json: bool,
     pub disable_cachebuster: bool,
     pub delay: Duration,
@@ -66,7 +66,6 @@ pub struct Config {
     pub learn_requests_count: usize,
     pub max: usize,
     pub concurrency: usize,
-    pub http2: bool,
     pub verify: bool,
     pub reflected_only: bool
 }
