@@ -59,7 +59,7 @@ pub fn compare(
     ) {
         Ok(val) => val,
         Err(err) => {
-            writeln!(io::stderr(), "Unable to compare: {}", err).ok();
+            writeln!(io::stderr(), "Unable to compare: {}", err).ok(); //TODO return error instead
             std::process::exit(1);
         }
     } {
@@ -118,6 +118,10 @@ pub fn fix_headers<'a>(header: &'a str) -> Option<String> {
 
     if RE.is_match(header) {
         Some(RE.replace_all(header, "").to_string())
+
+    // hyper throws an error in case the Content-Length header contains random value with http2
+    } else if header.to_ascii_lowercase() == "content-length" {
+        Some(String::from("disabled"))
     } else {
         None
     }
@@ -150,11 +154,8 @@ pub fn generate_request(config: &Config, initial_query: &HashMap<String, String>
     req.push(' ');
     req.push_str(&config.path.replace("%s", &query));
 
-    if config.http2 {
-        req.push_str(" HTTP/2\n");
-    } else {
-        req.push_str(" HTTP/1.1\n");
-    }
+
+    req.push_str(" HTTP/1.1\n");
 
     if !config.headers.keys().any(|i| i.contains("Host")) {
         req.push_str(&("Host: ".to_owned() + &config.host));
@@ -196,7 +197,10 @@ pub async fn generate_data(config: &Config, stats: &mut Statistic, client: &Clie
 
     writeln!(io::stdout(), "Request:\n{}", req).ok();
 
-    let response = request(config, stats, client, &query, 0).await;
+    let response =
+        request(config, stats, client, &query, 0)
+            .await
+            .expect("Unable to connect to the server");
 
     writeln!(
         io::stdout(),
@@ -363,6 +367,7 @@ pub fn parse_request(config: Config, proto: &str, request: &str, custom_paramete
     }
 
     //check whether the body type can be json
+     //TODO check whether can be combined with the same check in args.rs
     let body_type = if config.body_type.contains('-') && config.as_body && !custom_parameter_template
     && (
         content_type.contains("json") || (!body.is_empty() && body.starts_with('{') )
@@ -401,7 +406,6 @@ pub fn parse_request(config: Config, proto: &str, request: &str, custom_paramete
         body,
         body_type,
         parameter_template,
-        http2,
         initial_url,
         ..config
     })
